# -*- coding: utf-8 -*-
"""
Created on Tue Mar 26 08:13:26 2019

@author: RAMIAN
"""

from rskfd import instrumentRS


class instrumentRS_FSW(instrumentRS):      
    """Python class implementing functionality for Rohde & Schwarz FSW class Signal- And Spectrum Analyzer instruments.
    It is based on the instrumentRS class for socket connectivity of R&S instruments."""
    #Store all measurement objects in this dictionary
    __MeasurementList = {}
    __PersistentChannels = None
    __OverWriteChannels = None
    __CurrentChannel = "Spectrum"
    
    def __init__(self, IpAddress,Port=5025, timeout=2, PersistentChannels = True, OverWriteChannels = True):
        super(instrumentRS_FSW,self).__init__( IpAddress, Port, timeout)
        self.__PersistentChannels = PersistentChannels
        self.__OverWriteChannels = OverWriteChannels
        
    def ChannelSelect( self, channelname):
        if self.__CurrentChannel != channelname:
            self.__CurrentChannel = channelname
            self.Write( "INST:SEL '"+ self.__CurrentChannel+"'")
    
        
    def SetCenterFrequency( self, CenterFrequency):
        self.Write("FREQ:CENT "+str(CenterFrequency))
        
    def SetReferenceLevel( self, ReferenceLevel):
        self.Write("DISP:WIND:TRAC:Y:RLEV "+str(ReferenceLevel))
        
    def SetRfAttenuation( self, RfAttenuation):
        self.Write("INP:ATT:AUTO OFF")
        self.Write("INP:ATT "+str(RfAttenuation))
        
    def CoupleCenterFrequency( self):
        self.Write("INST:COUP:CENT ALL")
        
    def CoupleRefLevel( self):
        self.Write("INST:COUP:RLEV ALL")
        
    def CoupleRfAttenuation( self):
        self.Write("INST:COUP:ATTEN ALL")
        
        
    def SetupAclr( self, ChannelBandwidth = 98.28e6, AdjacentChannelBandwidth = 98.28e6, AdjacentChannelSpacing = 100e6, NumberOfAdjacentChannels = 1, Rbw = 100e3, CenterFrequency = 1e9, SweepTime = 10e-3):
        """Function that configures an ACLR measurement"""
    
        from AclrMeasurement import AclrMeasurement
        
        MeasName = "ACLR"
        #Need to remove existing object from MeasurementList expliticly, because of destructor
        self.__MeasurementList.pop(MeasName,None)
        self.__MeasurementList.update( {MeasName:AclrMeasurement( self, ChannelBandwidth = ChannelBandwidth, AdjacentChannelBandwidth = AdjacentChannelBandwidth,
                                                       AdjacentChannelSpacing = AdjacentChannelSpacing, NumberOfAdjacentChannels = NumberOfAdjacentChannels,
                                                       Rbw = Rbw, CenterFrequency = CenterFrequency, SweepTime = SweepTime, Overwrite = self.__OverWriteChannels, ChannelName = MeasName, Persistence = self.__PersistentChannels)})
    
    def MeasureAclr( self):
        """Function running the previously configured meausrement. Returns all measured values!"""
        
        MeasName = "ACLR"
        return self.__MeasurementList.get(MeasName).MeasureAclr()
    
    def DeleteAclr( self):
        MeasName = "ACLR"
        self.__MeasurementList.pop(MeasName)
    
    
    
    def SetupAmplifier(self, CenterFrequency):
        from AmplifierMeasurement import AmplifierMeasurement
        
        MeasName = "Amplifier"
        self.__MeasurementList.update( {MeasName:AmplifierMeasurement( self, CenterFrequency = CenterFrequency, Overwrite = self.__OverWriteChannels, Persistence = self.__PersistentChannels)})
        
    def GetAmplifier( self):
        MeasName = "Amplifier"
        return self.__MeasurementList.get(MeasName)

    def MeasureAmplifier(self):
        MeasName = "Amplifier"
        return self.__MeasurementList.get(MeasName).MeasureAmplifier()
    
    def DeleteAmplifier( self):
        MeasName = "Amplifier"
        self.__MeasurementList.pop(MeasName)

        
        
    def TakeScreenshot( self, localfile):
        """
        Function takes a screenshot from the instrument and transfers it to the local machine.
        
        Input arguments:
            localfile:  file name on local machine; file extension automatically sets file type

        Supported file types:
            png, jpg, wmf, bmp
        """
        import os

        remotefile = r"C:\temp\temp"
        # store current display update mode
        dispupdate = self.Query("SYST:DISP:UPD?")
        # display update on
        self.Write("SYST:DISP:UPD ON")
        # softkeys off
        self.Write("DISP:SKEY:STAT OFF")
        # logo off
        self.Write("DISP:LOGO OFF")
        # screencolor, no color change
        self.Write("HCOP:CMAP:DEF4")
        # No timestamp
        self.Write("HCOP:TDST:STAT OFF")
        # destination mass memory MMEM
        self.Write("HCOP:DEST 'MMEM'")
        # format
        _, fileextension = os.path.splitext( localfile)
        if fileextension.lower() == ".bmp":
            fileformat = "BMP"
        elif fileextension.lower() == ".wmf":
            fileformat = "WMF"
        elif fileextension.lower() == ".jpg":
            fileformat = "JPEG"
        elif fileextension.lower() == ".jpeg":
            fileformat = "JPEG"
        else:
            fileformat = "PNG"   

        self.Write("HCOP:DEV:LANG "+ fileformat)
        #self.Write("HCOP:CONT HCOP")
        self.Write("MMEM:NAME '"+remotefile+"'")
        self.Write("HCOP:IMM;*WAI")
        self.FileDownload( remotefile+"."+fileformat, localfile)
        self.Write("DISP:SKEY:STAT ON")
        self.Write( "SYST:DISP:UPD "+dispupdate)
        
        
    def SetupG5NR( self, CenterFrequency = 1e9, Downlink = True, Bandwidth = 100e6, NumberOfSubFrames = 10, CaptureTime = 20.1e-3):
        """Function that configures an ACLR measurement"""
    
        from G5NRMeasurement import G5NRMeasurement
        
        MeasName = "5GNR"
        #Need to remove existing object from MeasurementList expliticly, because of destructor
        self.__MeasurementList.pop(MeasName,None)
        self.__MeasurementList.update( {MeasName:G5NRMeasurement( self, Overwrite = self.__OverWriteChannels, ChannelName = MeasName, Persistence = self.__PersistentChannels,
                                                                 CenterFrequency = CenterFrequency,
                                                                 Downlink = Downlink, Bandwidth = Bandwidth, NumberOfSubFrames = NumberOfSubFrames, CaptureTime = CaptureTime)})
    
    def MeasureG5NR( self):
        """Function running the previously configured meausrement. Returns all measured values!"""
        
        MeasName = "5GNR"
        return self.__MeasurementList.get(MeasName).MeasureG5NR()
    
    def DeleteG5NR( self):
        MeasName = "5GNR"
        self.__MeasurementList.pop(MeasName)
        
        
    
    
def main():
    #for testing only, commented out are some things you can test!
    #pass

    #import time
    vse=instrumentRS_FSW("10.99.2.11",PersistentChannels =False)
    #vse.StartLogging("test.txt")
    vse.Connect()
    print(vse.GetID())
    vse.TakeScreenshot( "myscnshot.png")
    vse.TakeScreenshot( "myscnshot.bmp")
    vse.TakeScreenshot( "myscnshot.wmf")
    vse.TakeScreenshot( "myscnshot.jpg")
    
    
#ACLR Example
#    vse.SetupAclr(CenterFrequency = 3.3e9,NumberOfAdjacentChannels=3)
#    data=vse.MeasureAclr()
#    vse.DeleteAclr()
#    for x in data:
#        print(str(x))
#    vse.SetupAmplifier(3e9)


#mMplifier Example
    # vse.SetupAmplifier( 3e9)
    # ampli = vse.GetAmplifier()
    # ampli.ConnectGenerator( "169.254.138.159")
    # vse.MeasureAmplifier()
    
    #vse.FileDownload(r"C:\temp\FSW_Screenshot_2019-03-15_14-32-38.PNG",r"screenshot.png")
    #vse.FileUpload(r"screenshot.png", r"C:\temp\test.PNG")
    #print(vse.GetOpt())
    #vse.StartLogging("test.txt", True)
    #vse.Write("TRAC:IQ:DATA:FORM IQP")
    #vse.Write("FORM REAL,32;:TRAC:IQ:DATA:MEM?")
    #start = time.time_ns() only from 3.7 on
    #start = time.time()
    #floatdata=vse.ReadBinaryToFile("test1.iqw")
    #floatdata=vse.ReadBinary()
    #duration = time.time() - start
    #print("Total: {} samples in {:2.2f} ms. Speed: {:3.0f} MBit/s".format(len(floatdata),duration*1e3,len(floatdata)*4*8/duration/1e6))
    vse.GetSystErr()
    vse.Close()
    
    
if __name__ == "__main__":
    # execute only if run as a script
    pass
    #main()    



# -*- coding: utf-8 -*-
"""
Created on Fri Feb 22 22:01:25 2019

@author: RAMIAN
"""

#TODO: errorhandling


def __DataTypeStr2Int(DataType):
    """
    Converts a string data type to int, defaults to float 32
    float32 ==> 1, float64 ==> 2, int8 ==> 3, int16 ==> 4, int32 ==> 5
    """
    RetType = 1
    if DataType.lower().strip() == "float32":
        RetType = 1
    elif DataType.lower().strip() == "float64":
        RetType = 2
    elif DataType.lower().strip() == "int8":
        RetType = 3
    elif DataType.lower().strip() == "int16":
        RetType = 4
    elif DataType.lower().strip() == "int32":
        RetType = 5

    return RetType


def __DataTypeInt2Str(DataType):
    """
    Converts a string data type to int, defaults to float 32
    float32 ==> 1, float64 ==> 2, int8 ==> 3, int16 ==> 4, int32 ==> 5
    """
    RetType = "float32"
    if DataType == 1:
        RetType = "float32"
    elif DataType == 2:
        RetType = "float64"
    elif DataType == 3:
        RetType = "int8"
    elif DataType == 4:
        RetType = "int16"
    elif DataType == 5:
        RetType = "int32"

    return RetType


def __NumberOfBytes(DataType):
    """
    Returns number of bytes per value
    float32 ==> 4, float64 ==> 8, int8 ==> 1, int16 ==> 2, int32 ==> 4
    """
    RetVal = 4
    if not isinstance(DataType, int):
        DataType = __DataTypeStr2Int(DataType)
    if DataType == 1:
        RetVal = 4
    elif DataType == 2:
        RetVal = 8
    elif DataType == 3:
        RetVal = 1
    elif DataType == 4:
        RetVal = 2
    elif DataType == 5:
        RetVal = 4

    return RetVal


def Iqiq2Complex( iqData):
    """Returns a complex list of I/Q samples from a single list containing IQIQIQ values
    complexList = Iqiq2Complex(iqiqiqList)"""

    import logging

    if len(iqData) % 2 > 0:
        logging.warn("Expecting IQIQIQ order, input vector has odd number of samples!")

    NumberOfSamples = len(iqData) // 2

    complexList = [complex(iqData[2*n], iqData[2*n+1]) for n in range(NumberOfSamples)]    
    return complexList



def Iiqq2Complex(iqData):
    """Returns a complex list of I/Q samples from a single list containing IIIQQQ values
    complexList = Iiqq2Complex(iiiqqqList)"""

    import logging

    if len(iqData) % 2 > 0:
        logging.warn("Expecting IIIQQQ order, input vector has odd number of samples!")

    NumberOfSamples = len(iqData) // 2

    complexList = [complex(iqData[n], iqData[n+NumberOfSamples]) for n in range(NumberOfSamples)]    
    return complexList



def Complex2Iqiq(complexList):
    """Returns a list of I/Q samples from a complex list.
    iqiqiqList = Complex2Iqiq(complexList)"""

    f = lambda iq, i: iq.real if i == 0 else iq.imag
    iqiqiqList = [f(iq, i) for iq in complexList for i in range(2)]

    return iqiqiqList



def Complex2Iiqq( complexList):
    """Returns a list of I/Q samples from a complex list.
    iiiqqqList = Complex2Iiqq(complexList)"""

    iqiqiqList = [iq.real for iq in complexList]
    iqiqiqList.append([iq.imag for iq in complexList])

    return iqiqiqList



def WriteIqw( iqData, FileName, AppendExisting=False, DataType="float32"):
    """Writes an IQW file (file of binary floats).
    iqData can be a list of complex or list of floats. List of floats will
    be written directly into iqw - regardles of order iiqq or iqiq.
    Note: IIIQQQ is a deprecated format, don't use it for new files.
    writtenSamples = WriteIqw( iqList, "MyFile.iqiq.iqw").
    DataType (supported): float32, float64
    """

    import struct
    import logging

    # check if iqData is complex
    if isinstance(iqData[0], complex):
        iqData = Complex2Iqiq(iqData)

    NumberOfSamples = len(iqData) // 2
    ScalingFactor = 1

    try:
        if AppendExisting:
            file = open(FileName, "ab")
        else:
            file = open(FileName, "wb")
        if __DataTypeStr2Int(DataType) == 1:
            # float 32
            file.write(struct.pack("f"*len(iqData), *iqData))
        elif __DataTypeStr2Int(DataType) == 2:
            # float64
            file.write(struct.pack("d"*len(iqData), *iqData))
        elif __DataTypeStr2Int(DataType) == 3:
            # int16
            ScalingFactor = max(map(abs, iqData))/(2**(__NumberOfBytes(DataType)*8-1)-1)
            iqData = [round(item / ScalingFactor) for item in iqData]
            file.write(struct.pack("b"*len(iqData), *iqData))
        elif __DataTypeStr2Int(DataType) == 4:
            # int16
            ScalingFactor = max(map(abs, iqData))/(2**(__NumberOfBytes(DataType)*8-1)-1)
            iqData = [round(item / ScalingFactor) for item in iqData]
            file.write(struct.pack("h"*len(iqData), *iqData))
        elif __DataTypeStr2Int(DataType) == 5:
            # int16
            ScalingFactor = max(map(abs, iqData))/(2**(__NumberOfBytes(DataType)*8-1)-1)
            iqData = [round(item / ScalingFactor) for item in iqData]
            file.write(struct.pack("i"*len(iqData), *iqData))

        file.close
    except:
        logging.error("File (" + FileName + ") write error!")

    return NumberOfSamples, ScalingFactor



def ReadIqw( FileName, iqiq=True, DataType="float32"):
    """Reads an IQW (can be iiqq or iqiq) file. Returns complex samples.
    If iqiq is True, samples are read pairwise (IQIQIQ),
    otherwise in blocks, i first then q (IIIQQQ)
    Note: IIIQQQ is a deprecated format, don't use it for new files.
    iqList = ReadIqw("MyFile.iqw", iqiq = True)
    DataType (supported): float32, float64
    """

    import struct
    import logging

    BytesPerValue = __NumberOfBytes(__DataTypeStr2Int(DataType))

    try:
        file = open(FileName, "rb")
        data = file.read()
        file.close
        ReadSamples = len(data) // BytesPerValue
    except:
        logging.error("File open error (" + FileName+")!")

    if __DataTypeStr2Int(DataType) == 1:
        # float 32
        data = list(struct.unpack("f"*ReadSamples, data))
    elif __DataTypeStr2Int(DataType) == 2:
        # float64
        data = list(struct.unpack("d"*ReadSamples, data))
    elif __DataTypeStr2Int(DataType) == 3:
        # int8
        data = list(struct.unpack("b"*ReadSamples, data))
    elif __DataTypeStr2Int(DataType) == 4:
        # int16
        data = list(struct.unpack("h"*ReadSamples, data))
    elif __DataTypeStr2Int(DataType) == 5:
        # int32
        data = list(struct.unpack("i"*ReadSamples, data))

    if iqiq:
        data = Iqiq2Complex(data)
    else:
        data = Iiqq2Complex(data)

    return data



def WriteBin( iqData, fSamplingRate, FileName):
    """Writes a bin file, e.g. for use on KS generators
    iqData can be a list of complex or list of floats (iqiqiq format mandatory).
    writtenSamples = WriteBin("MyFile.bin",complexList, fs)
    Note: Max power is "1", i.e. I^2+Q^2<=1
    """

    import struct
    from datetime import date
    import math
    import logging

    # check if iqData is complex
    if isinstance(iqData[0], complex):
        iqData = Complex2Iqiq( iqData)

    NumberOfSamples = len(iqData) // 2

    # Find maximum magnitude and scale for max to be FullScale (1.0)
    power = []
    for n in range(NumberOfSamples):
        power.append(abs(iqData[2*n]**2 + iqData[2*n+1]**2))
    scaling = math.sqrt( max(power))

    # normalize to magnitude 1
    iqData = [ iq / scaling for iq in iqData]

    # calculate rms in dB (below full scale)
    rms = math.sqrt(sum(power)/NumberOfSamples)/scaling
    rms = abs(20*math.log10( rms))
    # Convert to int16, use floor function, otherwise distribution is not correct
    iqData = [ math.floor(iq * 32767 + .5) for iq in iqData]

    try:
        file = open( FileName, "wb")

        # binary block, big endian
        for nIdx in range(len(iqData)):
            file.write( struct.pack(">h", iqData[nIdx]))

        file.close()
    except:
        logging.error("File (" + FileName + ") write error!" )

    return NumberOfSamples



def WriteWv( iqData, fSamplingRate, FileName):
    """Writes a WV file.
    iqData can be a list of complex or list of floats (iqiqiq format mandatory).
    writtenSamples = WriteWv("MyFile.wv",complexList, fs)"""

    import struct
    from datetime import date
    import math
    import logging

    # check if iqData is complex
    if isinstance( iqData[0], complex):
        iqData = Complex2Iqiq( iqData)

    NumberOfSamples = len(iqData) // 2

    # Find maximum magnitude and scale for max to be FullScale (1.0)
    power = []
    for n in range(NumberOfSamples):
        power.append(abs(iqData[2*n]**2 + iqData[2*n+1]**2))
    scaling = math.sqrt(max(power))

    # normalize to magnitude 1
    iqData = [iq / scaling for iq in iqData]

    # calculate rms in dB (below full scale)
    rms = math.sqrt(sum(power)/NumberOfSamples)/scaling
    rms = abs(20*math.log10( rms))
    # Convert to int16, use floor function, otherwise distribution is not correct
    iqData = [ math.floor(iq * 32767 +.5) for iq in iqData]
        
    try:
        file = open( FileName, "wb")

        file.write( "{TYPE: SMU-WV,0}".encode("ASCII"))
        file.write( "{COMMENT: R&S WaveForm, TheAE-RA}".encode("ASCII"))
        file.write( ("{DATE: " + str(date.today())+ "}").encode("ASCII"))
        file.write( ("{CLOCK:" +str(fSamplingRate) + "}").encode("ASCII"))
        file.write( ("{LEVEL OFFS:" +  "{:2.4f}".format(rms) + ",0}").encode("ASCII"))
        file.write( ("{SAMPLES:" + str(NumberOfSamples) + "}").encode("ASCII"))
    #TODO: markers
    #     if( m1start > 0 && m1stop > 0)
    #        %Control Length only needed for markers
    #        fprintf(file_id,'%s',['{CONTROL LENGTH:' num2str(data_length) '}']);
    #        fprintf(file_id,'%s',['{CLOCK MARKER:' num2str(fSamplingRate) '}']);
    #        fprintf(file_id,'%s',['{MARKER LIST 1: ' num2str(m1start) ':1;' num2str(m1stop) ':0}']);
    #    end
        file.write( ("{WAVEFORM-" +  str(4*NumberOfSamples+1) + ": #").encode("ASCII"))

        # binary block
        file.write( struct.pack("h"*len(iqData),*iqData))
        
        file.write( "}".encode("ASCII"))

        file.close()
    except:
        logging.error("File (" + FileName +") write error!" )
        
    return NumberOfSamples



def ReadWv( FileName, ReadData=True, ConvertData=True):
    """Reads a WV file. Returns a list with complex numbers (I/Q) and the sampling rate
    iqiqiqList,fs = ReadWv("MyFile.wv")
    ReadData:       if set to False, we'll only return the parameters from the header
    ConvertData:    convert to complex floating point, otherwise keep original format"""

    import re
    import struct
    import logging
    from numpy import NaN

    try:
        file = open(FileName, "rb")
        if(ReadData):
            data = file.read()
        else:
            data = file.read( 30000)     # for the header, 30 kB should be sufficient
        file.close()
    except:
        logging.error( "File open error ("+ FileName+")!")
        return

    binaryStart = 0
    tags = ""
    Counter = 0
    ConverterSize = 20
    while (binaryStart == 0) & (Counter < len(data)):
        tags += data[Counter:Counter+ConverterSize].decode("ASCII", "ignore")
        Counter += ConverterSize
        #{WAVEFORM-20001: #
        res = re.search("WAVEFORM.{0,20}:.{0,3}#", tags)
        if res is not None:
            binaryStart = res.span()[1]

    if (Counter > len(data)) & (binaryStart == 0):
        logging.warning("Required tags not found, potentially incompatible file format!")

    res = re.search("SAMPLES[ ]*:[ ]*(?P<NumberOfSamples>[0-9]*)[ ]*}", tags)
    if res:
        NumberOfSamples = int(res.group("NumberOfSamples"))
    else:
        NumberOfSamples = NaN
    # res = re.search("CLOCK[ ]*:[ ]*(?P<SamplingRate>[0-9]*.[0-9]*[eE]?[+\-]?[0-9]*)",tags)
    res = re.search("CLOCK[ ]*:[ ]*(?P<SamplingRate>[0-9]*.[0-9]*[eE]?[+\-]?[0-9]*)[ ]*}", tags)
    if res:
        SamplingRate = float(res.group("SamplingRate"))
    else:
        SamplingRate = NaN
    res = re.search("LEVEL OFFS[ ]*:[ ]*(?P<RMSLevelOffset>[0-9]*.?[0-9]*),", tags)
    if res:
        RmsLevelOffset = float(res.group("RMSLevelOffset"))
    else:
        RmsLevelOffset = NaN
    res = re.search("Signal generated for SMx RMS level[ ]*:[ ]*(?P<RfRmsLevel>-?[0-9]*.?[0-9]*)[ ]*", tags)
    if res:
        RfRmsLevel = float(res.group("RfRmsLevel"))
    else:
        RfRmsLevel = NaN

    if ReadData and ConvertData:
        data = list(struct.unpack("h"*NumberOfSamples*2, data[binaryStart:binaryStart+NumberOfSamples*4]))
        data = list(map( lambda x: x/32767.0, data))
        data = Iqiq2Complex(data)
    else:
        data = data[binaryStart:]

    if ReadData:
        return data, SamplingRate
    else:
        return SamplingRate, NumberOfSamples, RmsLevelOffset, RfRmsLevel


def __WriteXml(fs, NumberOfSamples, filenameiqw, filenamexml, fCenterFrequency=0, DataType="float32", ScalingFactor=1):
    """Function to write the xml part of the iq.tar
    __WriteXml( samplingrate, numberofsamples, filenameiqw, filenamexml)
    DataType (supported): float32, float64
    ScalingFactor: scaling factor in Volts
    """

    from datetime import datetime

    xmlfile = open(filenamexml, "w")

    xmlfile.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n")
    xmlfile.write("<?xml-stylesheet type=\"text/xsl\" href=\"open_IqTar_xml_file_in_web_browser.xslt\"?>\n")
    xmlfile.write("<RS_IQ_TAR_FileFormat fileFormatVersion=\"2\" xsi:noNamespaceSchemaLocation=\"http://www.rohde-schwarz.com/file/RsIqTar.xsd\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">\n")
    # Optional
    xmlfile.write("<Name>Python iq.tar Writer (iqdata.py)</Name>\n")
    # Optional
    xmlfile.write("<Comment>RS WaveForm, TheAE-RA</Comment>\n")
    xmlfile.write("<DateTime>"+ datetime.now(None).isoformat() + "</DateTime>\n")
    xmlfile.write("<Samples>" + str(NumberOfSamples) + "</Samples>\n")
    xmlfile.write("<Clock unit=\"Hz\">" + str(fs) + "</Clock>\n")
    xmlfile.write("<Format>complex</Format>\n")
    xmlfile.write(f"<DataType>{__DataTypeInt2Str(__DataTypeStr2Int(DataType))}</DataType>\n")
    # Optional
    xmlfile.write(f"<ScalingFactor unit=\"V\">{ScalingFactor}</ScalingFactor>\n")
    # Optional
    # xmlfile.write("<NumberOfChannels>1</NumberOfChannels>\n")
    xmlfile.write("<DataFilename>" + filenameiqw + "</DataFilename>\n")
    # Optional
    if fCenterFrequency != 0:
        xmlfile.write( f'<UserData><RohdeSchwarz><SpectrumAnalyzer><CenterFrequency unit="Hz">{fCenterFrequency}</CenterFrequency></SpectrumAnalyzer></RohdeSchwarz></UserData>\n')

    xmlfile.write("</RS_IQ_TAR_FileFormat>\n")
    xmlfile.close()

    return



def WriteIqTar(iqData, fs, FileName, fCenterFrequency=0, DataType="float32"):
    """Writes an iq.tar file. Complex iqData values are interpreted as Volts.
    iqData can be a list of complex or list of floats (iqiqiq format).
    writtenSamples = WriteIqTar(iqList,fs,"MyFile.iq.tar")
    DataType (supported): float32, float64
    """

    import tarfile
    import os
    import re
    import logging

    ScalingFactor = 1
    path, filename = os.path.split(FileName)
    # Create binary file
    binaryfile = re.sub("iq.tar", f"complex.1ch.{__DataTypeInt2Str(__DataTypeStr2Int(DataType))}", filename, flags=re.IGNORECASE)
    NumberOfSamples, ScalingFactor = WriteIqw(iqData, os.path.join(path, binaryfile), DataType=DataType)
    if NumberOfSamples == 0:
        return 0

    # xsltfilename = "open_IqTar_xml_file_in_web_browser.xslt"
    xmlfilename = re.sub("iq.tar", "xml", filename, flags=re.IGNORECASE)
    __WriteXml(fs, NumberOfSamples, binaryfile, os.path.join(path, xmlfilename), fCenterFrequency=fCenterFrequency, DataType=DataType, ScalingFactor=ScalingFactor)

    try:
        tar = tarfile.open(FileName, "w")
        tar.add(os.path.join(path, binaryfile), arcname=binaryfile)
        # xslt is optional
        # tar.add( os.path.join(path, xsltfilename), arcname=xsltfilename)
        tar.add(os.path.join(path, xmlfilename), arcname=xmlfilename)
        tar.close()
        os.remove(os.path.join(path, binaryfile))
        os.remove(os.path.join(path, xmlfilename))
    except:
        logging.error("IqTar (" + FileName +") write error!" )

    return NumberOfSamples



def ReadIqTar(FileName, ChannelToRead=1):
    """Reads an iq.tar file. 
    data,fs = ReadIqTar("MyFile.iq.tar", ChannelToRead = 1)
    ChannelToRead specifies the channel to be read. "0" reads all channels and returns a matrix.
    """

    import tarfile
    import os
    import xml.etree.ElementTree as ET
    import logging

    data = []
    fs = 0

    try:
        tar = tarfile.open(FileName, "r:")
        a = tar.getnames()
        xmlfile = [filename for filename in a if ".xml" in filename.lower()]
        xmlfile = xmlfile[0]
        tar.extract(xmlfile)
        tree = ET.parse(xmlfile)
        root = tree.getroot()
        binaryfilename = root.find("DataFilename").text
        fs = float(root.find("Clock").text)

        helper = root.find("DataType")
        DataType = __DataTypeInt2Str(__DataTypeStr2Int(helper.text))

        helper = root.find("Samples")
        NumberOfSamples = 1
        if helper.text:
            NumberOfSamples = int(root.find("Samples").text)

        helper = root.find("ScalingFactor")
        ScalingFactor = 1
        if helper.text:
            if helper.get("unit") != "V":
                logging.warn("Only (V)olts scaling factor supported - assuming 1V!")
            else:
                ScalingFactor = float(root.find("ScalingFactor").text)

        helper = root.find("NumberOfChannels")
        NumberOfChannels = 1
        if helper is not None:
            if helper.text:
                NumberOfChannels = int(root.find("NumberOfChannels").text)

        os.remove(xmlfile)
        del root
        tar.extract(binaryfilename)
        tar.close()
        data = ReadIqw(binaryfilename, DataType=DataType)
        os.remove(binaryfilename)

    except:
        logging.error("IqTar (" + FileName + ") read error!")

    # Apply scaling factor
    if ScalingFactor != 1:
        data = [sample * ScalingFactor for sample in data]

    # separate channels
    if NumberOfChannels > 1:

        if (ChannelToRead > NumberOfChannels):
            logging.warn("File " + FileName + " only contains " + str(NumberOfChannels) + " channels, using channel 1!")
            ChannelToRead = 1
        if ChannelToRead > 0:
            data1 = []
            # for n in range(NumberOfSamples):
            # data1.append( data[n*NumberOfChannels+ChannelToRead-1])
            data1 = data[ChannelToRead-1::NumberOfChannels]
        else:
            import numpy
            data1 = numpy.empty((NumberOfSamples, NumberOfChannels), dtype=complex)
            for n in range(NumberOfSamples):
                for m in range(NumberOfChannels):
                    if m==0:
                        data1[n][m] = data[n*NumberOfChannels+m]
        data = data1

    return data, fs



def Iqw2Iqtar(FileName, fs, keepIqw=False, fCenterFrequency=0, DataType="float32"):
    """Converts an iqw file into iq.tar. Suggested to use after directly reading
    binary data from instrument into file (iqw).
    Note: iqw must be in iqiqiq format
    iqtarFilename = WriteIqTar(iqList,fs,"MyFile.iq.tar")"""

    import os
    import tarfile
    import re
    import logging

    NumberOfSamples = 0

    if os.path.isfile( FileName):
        NumberOfSamples = os.stat( FileName).st_size // 8
    else:
        logging.error("File " + FileName+" does not exist!")

    path, filename = os.path.split(FileName)
    iqtarfile = re.sub("iqw", "iq.tar", filename, flags=re.IGNORECASE)
    xmlfile = re.sub("iqw", "xml", filename, flags=re.IGNORECASE)
    binaryfile = re.sub("iqw", f"complex.1ch.{__DataTypeInt2Str(__DataTypeStr2Int(DataType))}", filename, flags=re.IGNORECASE)
    os.rename(FileName, os.path.join(path, binaryfile))

    __WriteXml(fs, NumberOfSamples, binaryfile, os.path.join(path, xmlfile), fCenterFrequency=fCenterFrequency, Datatype=DataType)

    try:
        tar = tarfile.open( os.path.join(path, iqtarfile), "w")
        tar.add(os.path.join(path, binaryfile), arcname=binaryfile)
        # xslt is optional
        # tar.add( os.path.join(path, xsltfilename), arcname=xsltfilename)
        tar.add(os.path.join(path, xmlfile), arcname=xmlfile)
        tar.close()
        if not keepIqw:
            os.remove(os.path.join(path, binaryfile))
        else:
            os.rename(os.path.join(path, binaryfile), FileName)
        os.remove(os.path.join(path, xmlfile))
    except:
        logging.error("IqTar (" + FileName + ") write error!")

    return os.path.join(path, iqtarfile)



def Iqw2Wv(FileName, fs, keepIqw=False):
    """Converts an iqw file into wv. Suggested to use after directly reading
    binary data from instrument into file (iqw).
    Note: iqw must be in iqiqiq format
    writtenSamples = WriteIqTar(iqList,fs,"MyFile.iq.tar")"""

    import os
    import re
    import logging
    import numpy
    import math
    import struct
    from datetime import date

    NumberOfSamples = 0
    BytesPerValue = 4
    BlockSize = 10000

    if os.path.isfile(FileName):
        NumberOfSamples = os.stat(FileName).st_size // (BytesPerValue * 2)
    else:
        logging.error("File " + FileName+" does not exist!")

    #path,filename = os.path.split(FileName)
    wvfile = re.sub("iqw", "wv", FileName, flags=re.IGNORECASE)

    rmsvalue = 0
    maxvalue = 0

    # first run throug iqw to determine rms and peak level  
    try:
        file = open( FileName, "rb")
        ReadCounter = 0
        while ReadCounter < NumberOfSamples:
            data = file.read(2*BytesPerValue*BlockSize)
            ReadSamples = len(data) // BytesPerValue
            ReadCounter += ReadSamples / 2
            data = list(struct.unpack("f"*ReadSamples, data))
            data = Iqiq2Complex(data)
            data = numpy.abs(data)
            rmsvalue += numpy.sum(numpy.power(numpy.abs(data), 2))
            maxofvector = numpy.amax(data)
            maxvalue = max(maxofvector, maxvalue)
        file.close
        rmsvalue = numpy.sqrt(rmsvalue/NumberOfSamples)
    except:
        logging.error( "File open error (" + FileName+")!")

    scaling = maxvalue
    maxvalue = 20*numpy.log10(maxvalue/scaling)
    rmsvalue = 20*numpy.log10(rmsvalue/scaling)

    # now we convert the data
    try:
        fileout = open(wvfile, "wb")     
        file = open(FileName, "rb")

        # header
        fileout.write("{TYPE: SMU-WV,0}".encode("ASCII"))
        fileout.write("{COMMENT: R&S WaveForm, TheAE-RA}".encode("ASCII"))
        fileout.write(("{DATE: " + str(date.today())+ "}").encode("ASCII"))
        fileout.write(("{CLOCK:" + str(fs) + "}").encode("ASCII"))
        fileout.write(("{LEVEL OFFS:" + "{:2.4f}".format(-1*rmsvalue) + "," + "{:2.4f}".format(maxvalue) + "}").encode("ASCII"))
        fileout.write(("{SAMPLES:" + str(NumberOfSamples) + "}").encode("ASCII"))
        fileout.write(("{WAVEFORM-" + str(4*NumberOfSamples+1) + ": #").encode("ASCII"))

        # now copy data from iqw to wv
        ReadCounter = 0
        while ReadCounter < NumberOfSamples:
            data = file.read(2*BytesPerValue*BlockSize)
            ReadSamples = len(data) // BytesPerValue
            ReadCounter += ReadSamples / 2
            data = list(struct.unpack("f"*ReadSamples, data))
            data = data / scaling
            # Convert to int16, use floor function, otherwise distribution is not correct
            data = [math.floor(iq * 32767 + .5) for iq in data]
            fileout.write(struct.pack("h"*len(data),*data))

        fileout.write("}".encode("ASCII"))
        fileout.close()
        file.close()

        # remove iqw
        os.remove(FileName)
    except:
        logging.error("File (" + FileName + ") write error!")    



if __name__ == "__main__":
    # execute only if run as a script
    pass



# -*- coding: utf-8 -*-
"""
Created on Tue Feb 19 15:02:17 2019

@authors: RAMIAN, TIPTON
"""

from enum import Enum
import math
import socket
import logging
import struct
import timeit
import time
import datetime

class instrument(object):
    '''
    Python class implementing a raw socket connection client,
    mainly designed around instrument control
    '''

    __ipaddress = ""
    __port = 5025
    __sendcls = True
    __msocket = None
    __timeout = 2
    __LogFile = None
    __LogReadings = True
    __TimeStamp = False
    # It's recommended to use a power of 2 for BufferSize
    _InBufferSize = 4* 2**10   #4kByte
    __IDString = ""
    __Options = None


    class SyncMethod(Enum):
        '''
        enum class supporting different sync methods
        '''
        OpcQuery = 0
        Blocking = 1
        StbPoll = 2


    @staticmethod
    def GetNumberOfDigits(number):
        '''
        count number of digits
        '''

        if number < 0:
            number *= -1
        if number == 0:
            return 1

        return math.ceil(math.log10( number))


    def __init__(self, IpAddress, port=5025, timeout=2, sendcls=True):
        '''
        init method;
            IP Address is required
            port            : port to communicate with
            timeout         : timeout value
            sendcls         : clear error queue before communicating to instrument
        '''
        self.__ipaddress = IpAddress
        self.__port = port
        self.__timeout = timeout
        self.__sendcls = sendcls


    def __del__(self):
        if self.__msocket:
            self.__msocket.close()
        if self.__LogFile:
            self.__LogFile.close()


    def __QueryID(self):
        self.Write("*IDN?")
        self.__IDString = self.Read()


    def GetID(self):
        '''
        return the instrument's ID string
        '''
        if self.__IDString == '':
            self.__QueryID()
        return self.__IDString


    def __QueryOpt(self):
        self.Write("*OPT?")
        self.__Options = self.Read().split(",")
        self.__Options.remove("")


    def GetOpt(self):
        '''
        return the instrument's option string
        '''
        if self.__Options is None:
            self.__QueryOpt()
        return self.__Options


    def IsLocalHost(self):
        '''
        Return boolean true when instrument runs on localhost (e.g. VSE)
        '''
        if self.__ipaddress.lower().find('localhost') >= 0 | self.__ipaddress.lower().find('127.0.0.1'):
            # missing check for same IP address
            return True
        else:
            return False


    def SetTimeout(self, timeout):
        '''
        Set the timeout for the connection (non blocking socket).
        For a blocking socket, use 'None'
        '''
        self.__timeout = timeout
        try:
            self.__msocket.settimeout( self.__timeout)
        except:
            pass



    def GetTimeout(self):
        '''
        Returns the specified timeout, 'None' meaning blocking socket
        '''
        return self.__msocket.gettimeout()


    def Synchronize(self, method=SyncMethod.StbPoll, timeout=None):
        '''Function to synchronize with instruments.
        Input:
        SyncMethod:
            SyncMethod.OpcQUery:    Use *OPC? to synchronize, i.e. a blocking query
            SyncMethod.Blocking:    Use *WAI to synchronize, i.e. block execution of next command until finished, no query
            SyncMethod.StbPoll:     Poll *ESR? until bit 0 is set, i.e. non blocking query
        timeout:    Timeout value, before function returns with timeout error

        Output:
        SyncOk: returns 1 when sync was successful, or 0 on timeout
        '''

        SyncOk = 0

        # Save current timeout and set user timeout
        if timeout is None:
            timeoutstorage = self.__timeout
            self.SetTimeout(timeout)
        else:
            timeout = self.GetTimeout()

        # OPC Query Sync mode
        if method == self.SyncMethod.OpcQuery:
            self.Write('*OPC?')
            try:
                value = int(self.Read())
                if value == 1:
                    SyncOk = 1
            except socket.timeout:
                pass

        # *WAI blocking mode
        if method == self.SyncMethod.Blocking:
            self.Write('*WAI')

        # *ESR? polling mode
        if method == self.SyncMethod.StbPoll:
            self.Write('*OPC')
            start = timeit.default_timer()
            while(not SyncOk and ((timeit.default_timer()-start) <= self.__timeout) ):
                SyncOk = int(self.Query('*ESR?')) & 1
                # sleep for 2 ms, so we don't poll too often
                time.sleep(.002)

        # Restore timeout
        if timeout != self.__timeout:
            self.SetTimeout(timeoutstorage)

        return SyncOk


    def Connect(self, getID=True, getOptions=False, Preset=False):
        '''
        Connects to the specified instrument and opens the connection
        '''

        try:
            self.__msocket = socket.socket( socket.AF_INET, socket.SOCK_STREAM)
            self.__msocket.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
            self.__msocket.connect( (self.__ipaddress, self.__port))
            self.__msocket.settimeout(self.__timeout)
            if Preset:
                self.Query('*RST;*OPC?')
            #Clear Error Queue
            if self.__sendcls:
                self.Write('*CLS')
            if getID:
                self.__QueryID()
            if getOptions:
                self.__QueryOpt()
        except ConnectionRefusedError:
            return -1
        except TimeoutError:
            return -1

        return 0


    def Close(self):
        '''
        Closes the connection
        '''

        self.CloseSocket()
        return


    def StartLogging(self, FileName, ClearFile=False, LogReadings=True, TimeStamp=False):
        '''
        Start logging into FileName
        ClearFile deletes file, if set to True
        '''
        FileMode = 'a'
        if ClearFile:
            FileMode = 'w'
        self.__LogFile = open(FileName, FileMode)
        self.__LogReadings = LogReadings
        self.__TimeStamp = TimeStamp


    def StopLogging(self):
        '''
        Stop logging
        '''
        self.__LogFile.close()
        self.__LogFile = None


    def UserCommentToLogFile(self, comment):
        '''
        Write a user comment to the log file
        '''
        comment = 'USERCOMMENT: ' + comment
        self.___LogToFile(comment)


    def ___LogToFile(self, Command):
        if self.__LogFile is None:
            return -1

        if self.__TimeStamp:
            self.__LogFile.write('{:%Y-%m-%d %H:%M:%S.%f}   '.format(datetime.datetime.now())+Command+"\n")
        else:
            self.__LogFile.write(Command+"\n")
        return 0


    def Write(self, Command, Binary=False, AddTermination=True):
        '''
        Write a command; termination character (\n) automatically added
        '''

        if self.__msocket is None:
            raise Exception('Connection is closed, use connect first!')

        self.___LogToFile( Command)
        if AddTermination:
            if Command.endswith("\n") is False:
                Command = Command + "\n"
        if Binary:
            self.__msocket.sendall(Command)
        else:
            self.__msocket.sendall(Command.encode('ASCII'))


    def Read(self, decode=True, NumberOfBytes=-1):
        '''
        Read back from the connected instrument
        '''

        if self.__msocket is None:
            raise Exception('Connection is closed, use connect first!')

        if NumberOfBytes == -1:
            NumberOfBytes = self._InBufferSize

        try:
            data = self.__msocket.recv(NumberOfBytes)
        except:
            message = "Timeout while reading data!"
            if self.__LogReadings:
                self.___LogToFile(message)
            else:
                logging.warn(message)
            return None

        if decode:
            try:
                data = data.decode("ASCII")
                if data.endswith("\n"):
                    data = data[:-1]
            except:
                message = "Binary data received!"
                if self.__LogReadings:
                    self.___LogToFile(message)
                else:
                    logging.warn(message)
                data = str(len(data)) + " binary bytes"

            if self.__LogReadings:
                self.___LogToFile(data)
        else:
            if self.__LogReadings:
                self.___LogToFile("{} bytes of binary data read".format(len(data))) 

        return data


    def Query(self, Command):
        """Write command and read result from connected instrument"""

        self.Write(Command)
        data = self.Read()

        return data


    def CloseSocket(self):
        """Close the connection"""
        self.__msocket.close()
        self.__IDString = ""
        self.__Options = []


    def OpenSocket( self):
        """Open the connection"""
        try:
            self.__msocket.connect((instrument.__ipaddress, instrument.__port))
            self.__msocket.settimeout(self.__timeout)
        except ConnectionRefusedError:
            pass


    def GetSystErr(self):
        """Return system error as (integer,string)"""
        self.Write("SYST:ERR?")
        result = self.Read()
        retlist = result.split(",")

        return int(retlist[0]), retlist[1]


    def UseNaN(self, vfcData):
        """Replace the SCPI representation of NaN (9.91*10^37) by NaN"""

        return [math.nan if x > 9.909e37 else x for x in vfcData]


    def ReadBinary(self, BytesPerValue=4, IqRead=False, NaNCheck=False, ReturnSamples=True):
        """Binary read of multiple values from the instrument, e.g. trace or I/Q data
        Input Values:
            BytesPerValue:  specifies the number of bytes that form a float; in format REAL,32 this is 4
            IqRead:         if set to true, complex I/Q pairs are returned; works only if instrument returns samples in iqiq order
            NaNCheck:       if set to true, the SCPI representation of NaN is replaced by Python's NaN
            ReturnSamples:  only set to false for interface tests; if set to false, function will only read back bytes and dump them

        Output Values:
            data vector when ReturnSamples is true
            byte counter when ReturnSamples is false
            """

        digits = self.Read(NumberOfBytes=2)

        if digits[1] == "(":
            # in this case we have more than 10 digits for the number of bytes enclosed by "(" and ")"
            digits += self.Read(NumberOfBytes=11)
            while digits[-1] != ")":
                digits += self.Read(NumberOfBytes=1)
            NumberOfBytesToRead = int(digits[2:-1])
        else:
            digits = int(digits[1])
            NumberOfBytesToRead = int(self.Read(NumberOfBytes=digits))

        if ReturnSamples:
            # This is the real code to read out samples
            NumericData = []
            oldData = b""

            ReadCounter = 0
            while ReadCounter <= NumberOfBytesToRead:   # we need to also read the closing character
                RawData = self.Read(decode=False)
                ReadCounter += len(RawData)
                RawData = oldData + RawData
                ReadSamples = len(RawData) // BytesPerValue
                if BytesPerValue == 4:
                    NumericData.extend(list(struct.unpack("f"*ReadSamples, RawData[0:ReadSamples*BytesPerValue])))
                elif BytesPerValue == 8:
                    NumericData.extend(list(struct.unpack("d"*ReadSamples, RawData[0:ReadSamples*BytesPerValue])))
                else:
                    logging.error("Format currently not supported!")
                oldData = RawData[ReadSamples*BytesPerValue:]

            if NaNCheck:
                NumericData = self.UseNaN( NumericData)

            if IqRead:
                from rskfd import Iqiq2Complex
                NumericData = Iqiq2Complex( NumericData)

            return NumericData
        else:
            # Binary Read and Dump results section; only a byte counter is returned; transfer speed check only
            ReadCounter = 0
            while ReadCounter <= NumberOfBytesToRead:   #we need to also read the closing character
                RawData = self.Read( decode=False)
                ReadCounter += len(RawData)

            return ReadCounter


    def ReadBinaryToFile(self, FileName):
        """Binary read of multiple values from the instrument directly into an iqw file.
        This function assumes 4 byte floating point format, usually commanded by
        FORM REAL,32"""

        digits = self.Read(NumberOfBytes=2)

        if digits[1] == "(":
            # in this case we have more than 10 digits for the number of bytes enclosed by "(" and ")"
            digits += self.Read(NumberOfBytes=11)
            while digits[-1] != ")":
                digits += self.Read(NumberOfBytes=1)
            NumberOfBytesToRead = int(digits[2:-1])
        else:
            digits = int(digits[1])
            NumberOfBytesToRead = int(self.Read(NumberOfBytes=digits))

        file = open(FileName, "wb")

        ReadCounter = 0
        while ReadCounter <= NumberOfBytesToRead:   # we need to also read the closing character
            RawData = self.Read(decode=False)
            ReadCounter += len(RawData)
            if ReadCounter <= NumberOfBytesToRead:
                file.write(RawData)
            else:
                file.write(RawData[:(NumberOfBytesToRead-ReadCounter)])

        file.close()


    def Reset(self):
        """Reset on the connection, includes sending an ABORt, a *CLS
        Close and Reconnect."""
        self.Write("ABORt;*CLS")
        self.CloseSocket()
        self.Connect()



    def EmptyBuffer(self):
        """Empty raw socket buffer. Work around needed if program terminated during raw socket transfer."""

        actualtimeout = self.GetTimeout()
        self.SetTimeout(.1)
        try:
            while self.Read(decode=False, NumberOfBytes=1000):
                pass
        except:
            pass
        self.SetTimeout(actualtimeout)


if __name__ == "__main__":
    # execute only if run as a script
    fsw = instrument('localhost', port=5026, sendcls=False)
    fsw.Connect()
    fsw.Query('*IDN?')
    fsw.Close()
    # pass   



# -*- coding: utf-8 -*-
"""
Created on Sat Jul 11 2020

(C) 2020, Florian Ramian
"""

from enum import Enum 
class unit(Enum):
    volt    = 1
    dBm     = 2
    watt    = 3



def ShowLogFFT( data, fs = 1, MaxFftLength = 10000, bPlotRelative = True):
    """Helper function showing the magnitude of the FFT on a logarithmic scale.
    Signal length is limited to MaxFftLength to speed up FFT calculation.
    Input data is interpreted as Volts and converted to dBm."""

    # currently disabled
    # pass

    import matplotlib.pyplot as plt
    import numpy
    limiterlen = min(MaxFftLength,len(data))

    plt.ion()
    plt.show()

    if bPlotRelative:
        mag = 20*numpy.log10(numpy.abs(numpy.fft.fftshift(numpy.fft.fft(data[:limiterlen]))))
        # normalize to mean
        meanmag = 10*numpy.log10( numpy.mean( numpy.power( 10, mag/10)))
        mag = mag - meanmag
    else:
        mag = 10*numpy.log10(numpy.power( numpy.abs( numpy.fft.fftshift( numpy.fft.fft( data[:limiterlen]))), 2)/50/limiterlen)+30

    freq = numpy.multiply(range(-(limiterlen//2), (limiterlen+1)//2), fs/limiterlen)
    plt.plot(freq,mag)
    plt.grid( True)
    plt.xlabel( 'freq / Hz')
    if bPlotRelative:
        plt.ylabel( 'relative power / dB')
    else:
        plt.ylabel( 'power / dBm')
    plt.draw()
    plt.pause(.001)



def ConvertUnit( data, inUnit=unit.volt, outUnit=unit.dBm, impedance = 50):
    """
    ConverUnit converts a vector or scalar from an input unit to an output unit using the given impedance.
    data:       input vector or scalar; values can be complex for volts
    inUnit:     input unit, default Volts
    outUnit:    output unit, default dBm
    impedance:  impedance for conversion from power to amplitude, default 50 Ohms
    """
    
    # Not the fastest implementation, but easiest:
    # convert all input to volts, then to target unit
    import numpy
    
    if( inUnit == unit.watt):
        data = numpy.sqrt(data*impedance)
    elif( inUnit == unit.dBm):
        data = numpy.sqrt( numpy.power( 10, data/10) / 1000 * impedance)

    # here we have volts

    if( outUnit == unit.dBm):
        data = 10 * numpy.log10( numpy.power( numpy.abs( data), 2) / impedance) + 30
    elif( outUnit == unit.watt):
        data = numpy.power( numpy.abs( data), 2) / impedance

    return data



def Scale2dBm( vfcIqData, fTargetRMSPower):
    '''
    Scales an input data vector to a target mean power
    vfcIqData:          input vector, any unit
    fTargetRMSPower:    target RMS power
    
    Output vector will be in Volts over 50 Ohms
    '''
    import numpy
        
    fInputPower = MeanPower( vfcIqData)
    vfcIqData = numpy.multiply( vfcIqData, 10**((fTargetRMSPower-fInputPower)/20))

    return vfcIqData



def MeanPower( data, inUnit=unit.volt, outUnit=unit.dBm, impedance = 50):
    """
    MeanPower calculates the mean power of an input vector
    data:       input vector
    inUnit:     input unit, default Volts
    outUnit:    output unit, default dBm
    impedance:  impedance for conversion from power to amplitude, default 50 Ohms
    """

    import numpy

    data = ConvertUnit( data, inUnit=inUnit, outUnit=unit.volt, impedance=impedance)

    # formula calculates mean power from volts to dBm
    power = numpy.log10( numpy.mean( numpy.power( numpy.abs(data), 2) / impedance)) * 10 +30

    power = ConvertUnit( power, inUnit=inUnit.dBm, outUnit=outUnit, impedance=impedance)
    
    return power



def MaxPower( data, inUnit=unit.volt, outUnit=unit.dBm, impedance = 50):
    """
    MaxPower returns the max power of an input vector
    data:       input vector
    inUnit:     input unit, default Volts
    outUnit:    output unit, default dBm
    impedance:  impedance for conversion from power to amplitude, default 50 Ohms
    """

    import numpy

    data = ConvertUnit( data, inUnit=inUnit, outUnit=unit.volt, impedance=impedance)

    # formula calculates mean power from volts to dBm
    power = numpy.log10( numpy.power( max( numpy.abs( data)), 2) / impedance) * 10 +30

    power = ConvertUnit( power, inUnit=inUnit.dBm, outUnit=outUnit, impedance=impedance)
    
    return power



if __name__ == "__main__":
    # execute only if run as a script
    pass



# -*- coding: utf-8 -*-
'''
Created on Thu Mar 21 15:04:00 2019

@author: RAMIAN
'''


import os
from functools import partial
from rskfd import instrument


class instrumentRS(instrument):
    '''
    Python class implementing functionality for Rohde & Schwarz instruments.
    It is based on the instrument class for socket connectivity.
    '''


    def FileDownload( self, remoteFile, localFile):
        '''
        Download file from the instrument to the local file
        '''
        self.Write(f'MMEM:DATA? "{remoteFile}"' )
        self.ReadBinaryToFile(localFile)


    def FileUpload( self, localFile, remoteFile):
        '''
        Upload a file to the instrument
        '''

        if os.path.isfile( localFile) == 0:
            raise Exception('File '+localFile+' does not exist')

        statinfo = os.stat(localFile)
        self.Write('MMEM:DATA "' + remoteFile + '",#' + str(instrument.GetNumberOfDigits(statinfo.st_size)) + str(statinfo.st_size), AddTermination = False )

        file = open( localFile, 'rb')
        for chunk in iter(partial(file.read, self._InBufferSize), b''):
            self.Write( chunk, Binary = True, AddTermination = False)
        file.close()
        self.Write('\n')


if __name__ == '__main__':
    # execute only if run as a script
    pass



# -*- coding: utf-8 -*-
"""
Created on Wed Jul 08 2020

(C) 2020, Florian Ramian
"""

# This is example code on how to use the instrument.py and derived classes

def exampleTestConnection( ipaddress):
    # use only the raw instrument.py interface and give all (SCPI) commands explicitly
    # this should work for all instruments supporting a raw socket interface
    # import instrument.py class
    import rskfd

    # now open connection, default port is 5025
    myinstrument = rskfd.instrument( ipaddress)
    myinstrument.Connect()

    # Write something "*IDN?"
    myinstrument.Write( "*IDN?")
    # Print whatever we read back
    print( myinstrument.Read())

    #Close connection
    myinstrument.CloseSocket()



def exampleFileDownload( ipaddress):
    # now we'll use instrumentRS class
    #import class

    import rskfd

    # now open connection, default port is 5025, same as above, but called now through instrumentRS
    myinstrument = rskfd.instrumentRS( ipaddress)
    myinstrument.Connect()

    # Write something "*IDN?", as above in example 1
    myinstrument.Write( "*IDN?")
    # Print whatever we read back
    print( myinstrument.Read())

    # by using instrumentRS - we get more functionality, e.g. FileDownload
    # uses raw string to avoid escaping, transfers a remote (win.ini) to a local file mytest.txt
    myinstrument.FileDownload( r"C:\Windows\win.ini", "mytest.txt")

    #Close connection
    myinstrument.CloseSocket()



def exampleCaptureIqData( ipaddress, CaptureLength=200000):
    # now we'll use instrumentRS.py
    #import class

    import rskfd

    # now open connection, default port is 5025, same as above, but called now through instrumentRS
    myinstrument = rskfd.instrumentRS( ipaddress)
    myinstrument.Connect()

    # Identify instrument
    print( myinstrument.Query( '*IDN?'))

    # now we'll set up a fill I/Q capture
    # Example would work on R&S signal and spectrum analyzers

    # Preset
    myinstrument.Write( '*RST')
    # open I/Q analyzer app
    myinstrument.Write( 'INST:SEL IQ')
    # single sweep mode
    myinstrument.Write( 'INIT:CONT OFF')
    # set frequency and reference level
    myinstrument.Write( 'FREQ:CENT 1.2e9')
    myinstrument.Write( 'DISP:WIND:TRAC:Y:RLEV 0')
    # set sampling rate and capture length
    fs = 10e6
    myinstrument.Write( 'TRAC:IQ:SRAT ' + str(fs))
    myinstrument.Write( 'TRAC:IQ:RLEN ' + str(CaptureLength))
    # now sweep and synchronize to end of sweep
    myinstrument.Write( 'INIT:IMM')
    myinstrument.Synchronize()
    # Read back iq vector, use binary transfer, IQ pairwise
    myinstrument.Write( 'FORM REAL,32;:TRAC:IQ:DATA:FORM IQP')
    myinstrument.Write( 'TRAC:IQ:DATA:MEM?')
    iq = myinstrument.ReadBinary()

    #Close connection
    myinstrument.CloseSocket()

    # iq vector is now 2xrecord length with I and Q pairwise
    # now we use iqdata module and convert it to a complex vector

    iq_complex = rskfd.Iqiq2Complex( iq)
    iq_complex.clear()

    # or we could write the data to an iq..pytar file (R&S analysis file format)
    rskfd.WriteIqTar( iq, fs, 'mytest.iq.tar' )



def exampleIqFileReadWrite():
    # this example generates a noise signal and saves/rereads it
    import rskfd
    import re
    from scipy import signal
    import numpy

    # create WGN signal with a mean power of -50 dBm
    noise = rskfd.CreateWGNSignal( -50)
    out = rskfd.LowPassFilter( noise, RelativeBandwidth=.8, FilterTaps=100, KeepPower=True)

    # create wv file, set sampling rate 125 MHz
    fs = 125e6
    filename = 'wgn_'+str(fs*.8/1e6)+'mhz'
    rskfd.WriteIqTar( out, fs, filename+'.iq.tar')

    # Read back wv and save it as iq.tar
    rereadnoise,rereadfs = rskfd.ReadIqTar( filename+'.iq.tar')
    # note that wv files are normalized to peak power, whereas iq.tar files are saved in Volts (typically)
    rskfd.WriteWv( rereadnoise, rereadfs, filename+'.wv')
    
    # plot noise
    rskfd.ShowLogFFT( noise, fs, bPlotRelative=False)
    rskfd.ShowLogFFT( out, fs, bPlotRelative=False)
    input( "Press return to finish!")


def exampleSpeedTest( ipaddress):
    # example that shows performance of I/Q data transfer
    
    import time
    import rskfd

    # use example3 to configure I/Q analyzer
    exampleCaptureIqData( ipaddress, CaptureLength=10e6)

    # now reopen connectio0
    myinstrument = rskfd.instrumentRS( ipaddress)
    myinstrument.Connect()

    # do a first measurement
    myinstrument.Write( 'TRAC:IQ:DATA?')
    iq = myinstrument.ReadBinary()

    meas_start = time.perf_counter()
    myinstrument.Write( 'TRAC:IQ:DATA?')
    iq = myinstrument.ReadBinary(IqRead=True)
    meastime = time.perf_counter() - meas_start
    print( "Single Meas & Read command (TRAC:IQ:DATA?)")
    print("Total: {} samples in {:2.2f} ms. Rate: {:2.2f} Mbit/s".format(len(iq),meastime*1e3,8*8*len(iq)/1e6/meastime))

    meas_start = time.perf_counter()
    myinstrument.Write( 'INIT:IMM;*WAI;:TRAC:IQ:DATA:MEM?')
    iq = myinstrument.ReadBinary(IqRead=True)
    meastime = time.perf_counter() - meas_start
    print( "Separate Meas & Read Commands (INIT:IMM;*WAI;:TRAC:IQ:DATA:MEM?)")
    print("Total: {} samples in {:2.2f} ms. Rate: {:2.2f} Mbit/s".format(len(iq),meastime*1e3,8*8*len(iq)/1e6/meastime))

    meas_start = time.perf_counter()
    myinstrument.Write( 'TRAC:IQ:DATA:MEM?')
    iq = myinstrument.ReadBinary(IqRead=True)
    meastime = time.perf_counter() - meas_start
    print( "Read only (TRAC:IQ:DATA:MEM?)")
    print("Total: {} samples in {:2.2f} ms. Rate: {:2.2f} Mbit/s".format(len(iq),meastime*1e3,8*8*len(iq)/1e6/meastime))

    # This code only transfers the bytes from instrument to PC - no conversion so it measures the interface speed
    meas_start = time.perf_counter()
    myinstrument.Write( 'TRAC:IQ:DATA:MEM?')
    samples = myinstrument.ReadBinary(ReturnSamples=False)
    meastime = time.perf_counter() - meas_start
    print( "Read only (TRAC:IQ:DATA:MEM?)")
    print("Total: {} bytes in {:2.2f} ms. Rate: {:2.2f} Mbit/s".format(samples,meastime*1e3,8*samples/1e6/meastime))

    myinstrument.CloseSocket()
   


def main():
    # Adapt ip address and uncomment the examples you want to run
    ipaddress = "10.99.2.11"
    # exampleTestConnection( ipaddress)
    # exampleFileDownload( ipaddress)
    # exampleCaptureIqData( ipaddress)
    # exampleIqFileReadWrite()
    exampleSpeedTest( ipaddress)


if __name__ == "__main__":
    main()


# -*- coding: utf-8 -*-
"""
Created on Sun Dec 12 2021

(C) 2021, Rohde&Schwarz, ramian
"""

import struct
import re
import math


def ConvertWv2BinStream( FileName, bScale = False):
    '''
    Convert large wv files to bin (KS file format), i.e. do not read full iq vector
    Note: bin file must not have a higher power than "1", use rescale option to make sure
    '''
    SampleSize = 4 # 2 x 2 bytes
    BlockSize = 50000

    try:
        file = open(FileName, "rb")
        data = file.read( 30000)     # for the header, 30 kB should be sufficient
    except:
        print( "File open error ("+ FileName+")!")

    binaryStart = 0
    tags = ""
    Counter = 0
    ConverterSize = 20
    while (binaryStart == 0) & (Counter < len(data)):
        tags += data[Counter:Counter+ConverterSize].decode("ASCII","ignore")
        Counter += ConverterSize
        #{WAVEFORM-20001: #
        res = re.search("WAVEFORM.{0,20}:.{0,3}#",tags)
        if res is not None:
            binaryStart = res.span()[1]

    if binaryStart == 0:
        print("Binary start not found!\n")

    if (Counter > len(data)) & (binaryStart == 0):
        print( "Required tags not found, potentially incompatible file format!")

    res = re.search("SAMPLES[ ]*:[ ]*(?P<NumberOfSamples>[0-9]*)[ ]*}",tags)
    if res:
        NumberOfSamples = int( res.group("NumberOfSamples"))
    else:
        NumberOfSamples = -1
    
    outFileName = FileName.lower().replace( '.wv', '.bin')
    outfile = open( outFileName, "wb")

    # data = data[binaryStart:]
    # data = data + file.read( SampleSize-(len(data) % SampleSize))
    # NumberOfInt16s = len(data)//2
    # SampleCount = NumberOfInt16s // 2
    # data = list(struct.unpack("h"*NumberOfInt16s, data))
    # for nIdx in range(NumberOfInt16s):
    #     if bScale:
    #         outfile.write( struct.pack(">h",data[nIdx]))
    fScaler = 1
    if bScale:
        # first loop only to find max
        SampleCount = 0
        file.seek(binaryStart)
        fMaxPower = 0
        while SampleCount < NumberOfSamples:
            SamplesToRead = NumberOfSamples-SampleCount
            if SamplesToRead > BlockSize:
                SamplesToRead = BlockSize
            data = file.read( SamplesToRead * SampleSize)
            NumberOfInt16s = len(data)//2
            data = list(struct.unpack("h"*NumberOfInt16s, data))
            fData = list(map( lambda x: x/32767.0, data ))
            for nIdx in range(0,len(fData)//2):
                fMaxPower = max( [fMaxPower, fData[nIdx*2]**2 + fData[1+nIdx*2]**2])
            SampleCount = SampleCount + SamplesToRead
        fScaler = math.sqrt( fMaxPower)
    
    SampleCount = 0
    file.seek(binaryStart)
    fMaxPower = 0
    while SampleCount < NumberOfSamples:
        SamplesToRead = NumberOfSamples-SampleCount
        if SamplesToRead > BlockSize:
            SamplesToRead = BlockSize
        data = file.read( SamplesToRead * SampleSize)
        NumberOfInt16s = len(data)//2
        data = list(struct.unpack("h"*NumberOfInt16s, data))
        for nIdx in range(NumberOfInt16s):
            if bScale:
                data[nIdx] = round( data[nIdx] / fScaler)
            outfile.write( struct.pack(">h",data[nIdx]))
        SampleCount = SampleCount + SamplesToRead

    file.close()
    outfile.close()



if __name__ == "__main__":
    # pass
    # Testcode
    filename = r'C:\Users\ramian\Downloads\EVM_Eval_11be_320MHz_MCS13'
    ConvertWv2BinStream( filename+'.wv', bScale=True)
    # import rskfd
    # iq,fs = rskfd.ReadWv(filename + '.wv')
    # rskfd.WriteBin( iq,fs, filename +'.bin')



# Public functions
from .remote_control.instrument import instrument
from .remote_control.instrumentRS import instrumentRS
from .remote_control.instrumentRS_FSW import instrumentRS_FSW
from .remote_control.instrumentRSExceptions import InstrumentError

from .iq_data_handling.iqdata import Iqiq2Complex
from .iq_data_handling.iqdata import Iiqq2Complex
from .iq_data_handling.iqdata import Complex2Iqiq
from .iq_data_handling.iqdata import Complex2Iiqq
from .iq_data_handling.iqdata import WriteIqw
from .iq_data_handling.iqdata import ReadIqw
from .iq_data_handling.iqdata import WriteBin
from .iq_data_handling.iqdata import WriteWv
from .iq_data_handling.iqdata import ReadWv
from .iq_data_handling.iqdata import WriteIqTar
from .iq_data_handling.iqdata import ReadIqTar
from .iq_data_handling.iqdata import Iqw2Iqtar
from .iq_data_handling.iqdata import Iqw2Wv

from .snp_handling.snpfiles import WriteSnP
from .snp_handling.snpfiles import ReadSnP

from .signal_generation.signal_generation import CreateWGNSignal
from .signal_generation.signal_generation import LowPassFilter

from .helper.helper import ShowLogFFT
from .helper.helper import MeanPower
from .helper.helper import MaxPower
from .helper.helper import ConvertUnit
from .helper.helper import Scale2dBm


# -*- coding: utf-8 -*-
"""
Created on Fri Mar 22 13:32:35 2019

@author: RAMIAN
"""

class InstrumentError(Exception):
    """Exception raised for errors in the input.

    Attributes:
        expression -- input expression in which the error occurred
        message -- explanation of the error
    """

    def __init__(self, message):
        self.message = message

class FileLoadError(Exception):
    """Exception raised for file load fails.

    Attributes:
        expression -- input expression in which the error occurred
        message -- explanation of the error
    """

    def __init__(self, message):
        self.message = message


# -*- coding: utf-8 -*-
"""
Created on Fri Mar 12 2021

(C) 2021, Rohde&Schwarz, ramian
"""

def ConvertSparams( s, inputformat = "ri", outputformat = "ri"):
### Helper function to convert s parameters from one format into another. Supported formats for inputformat and outputformat are:
    #           dbmag:  magnitude in dB/phase in degrees
    #           ri:     real / imaginary
    #           mag:    linear magnitude / degrees
###
    import numpy
    import logging

    if inputformat.lower().find( "ri")>=0:
        informat = 1
    elif inputformat.lower().find( "dbmag")>=0:
        informat = 2
    elif inputformat.lower().find( "mag")>=0:
        informat = 3
    else:
        logging.error( "Input format not supported!")
        return

    if outputformat.lower().find( "ri")>=0:
        outformat = 1
    elif outputformat.lower().find( "dbmag")>=0:
        outformat = 2
    elif outputformat.lower().find( "mag")>=0:
        outformat = 3
    else:
        logging.error( "Output format not supported!")
        return

    outvector = []
    try:
        vectorlength = len( s)
    except:
        vectorlength = 1
    for m in range( vectorlength):
        if numpy.isscalar( s):
            allsparams = s
        else:
            allsparams = s[m]

        outvectorentry = []
        try:
            numparamstowrite = len( s[0])
        except:
            numparamstowrite = 1

        for n in range( numparamstowrite):
            if numpy.isscalar( allsparams):
                sparam = allsparams
            else:
                sparam = allsparams[n]

            # Convert to RI first
            if informat == 1:
                helper = sparam
            elif informat == 2:
                helper = numpy.power( 10, numpy.real( sparam)/20) * numpy.exp( 1j * numpy.imag(sparam)/180*numpy.pi)
            else:
                helper = numpy.real( sparam) * numpy.exp( 1j * numpy.imag(sparam)/180*numpy.pi)

            # Convert to RI first
            if outformat == 1:
                sparam = helper
            elif outformat == 2:
                sparam = complex( 20 * numpy.log10( numpy.abs( helper)), numpy.angle( helper, deg=True))
            else:
                sparam = complex( numpy.abs( helper), numpy.angle( helper, deg=True))
            

            # make sure we always return a list of lists
            # if numpy.isscalar( sparam) and numparamstowrite == 1:
            #     outvectorentry = sparam       
            # else:
            outvectorentry.append( sparam)      

        if vectorlength > 1:
            outvector.append( outvectorentry)
        else:
            outvector = outvectorentry


    return outvector


def ReadSnP( filename, outputformat = "ri"):
    ### Read in a Touchstone file and return a S-parameter matrix (over frequency) as well as a frequency vector.
    #       outputformat:    format of output list <s>
    #           dbmag:  magnitude in dB/phase in degrees
    #           ri:     real / imaginary
    #           mag:    linear magnitude / degrees
    ###
    import logging
    import re
    import numpy

    s =  []
    f = []

    numparamstoread = 0
    if filename.lower().find(".s1p") >=0 or filename.lower().find(".fres") >=0:
        numparamstoread = 1
    elif filename.lower().find(".s2p") >=0:
        numparamstoread = 4
    elif filename.lower().find(".s3p") >=0:
        numparamstoread = 9
    elif filename.lower().find(".s4p") >=0:
        numparamstoread = 16
    else:
        logging.error( "Format not supported!")
        return

    searchstring = r"[+-]? *(?:\d+(?:\.\d*)?|\.\d+)(?:[eE][+-]?\d+)?"

    file = open( filename)

    importstyle = "ri"
    freqscaling = 1

    while True:
        inline = file.readline()
        if not inline:
            break
        
        if inline[0] == "!" or inline[0] == "#":
            if re.search( '[ ,\t]+RI[ ,\t]+', inline):
                importstyle = "ri"
            elif re.search( '[ ,\t]+DB[ ,\t]+', inline):
                importstyle = "dbmag"
            elif re.search( '[ ,\t]+MA[ ,\t]+', inline):
                importstyle = "mag"
            elif re.search( '[ ,\t]+HZ[ ,\t]+', inline):
                freqscaling = 1
            elif re.search( '[ ,\t]+KHZ[ ,\t]+', inline):
                freqscaling = 1e3
            elif re.search( '[ ,\t]+MHZ[ ,\t]+', inline):
                freqscaling = 1e6
            continue
        else:
            myvalues = [float(numbers) for numbers in re.findall(searchstring, inline)]

            if len(myvalues) != numparamstoread*2+1:
                #error
                logging.error( "Read error!")
                break
            else:
                f.append( myvalues[0] * freqscaling)
                # 1 sparameter ==> list of complex floats, otherwise list of lists
                if numparamstoread == 1:
                    helper = ConvertSparams( complex( myvalues[1], myvalues[2]), inputformat=importstyle)
                    if numpy.isscalar( helper) == False:
                        helper = helper[0]
                    s.append( helper)
                else:
                    sparamtupel = []
                    for n in range(numparamstoread):
                        helper = ConvertSparams( complex( myvalues[2*n+1], myvalues[2*n+2]), inputformat=importstyle)
                        if numpy.isscalar( helper) == False:
                            helper = helper[0]
                        sparamtupel.append( helper)
                    s.append( sparamtupel)
    

    file.close()

    return s,f


def WriteSnP( s, f, filename, inputformat = "ri", outputformat = "ri"):
    ### Write a Tocuhstone file, which contains frequency points of input vector f, and s-parameters contained in s:
    #       s:  list of n-tupels, where n is 1 for s1p or fres formats, and 4 for s2p
    #       f:  list of frequencies
    #       filename:   full filename
    #       inputformat:    format of input list <s>
    #       outputformat:   format in SnP file
    #           dbmag:  magnitude in dB/phase in degrees
    #           ri:     real / imaginary
    #           mag:    magnitude linear / degrees
    ###

    import logging
    import datetime
    import numpy
    import re

    #outputformat = "ri

    s = ConvertSparams( s, inputformat=inputformat, outputformat=outputformat) 

    file = open( filename, "w")

    if outputformat.lower().find("ri") >=0:
        file.write( "#  HZ   S   RI   R     50.00\n")
    elif outputformat.lower().find("dbmag") >=0:
        file.write( "#  HZ   S   DB   R     50.00\n")
    else:
        file.write( "#  HZ   S   MA   R     50.00\n")

    now = datetime.datetime.now()

    file.write( "! rskfd SnP writer tool\n")
    file.write( "! Date: " +  now.strftime("%Y-%m-%d %H:%M:%S") + "\n")

    try:
        numparamstowrite = len( s[0])
    except:
        numparamstowrite = 1


    #fileformats
    #   1: s2p
    #   2: fres
    #   0: s1p

    if filename.lower().find( ".s2p") >=0:
        fileformat = 1
    elif filename.lower().find( ".fres") >=0:
        fileformat = 2
    elif filename.lower().find( ".s1p") >=0:
        fileformat = 0
    else:
        logging.error( "File type not supported!")
        return

    if fileformat == 1:
        if numparamstowrite == 1:
            file.write( "! Measurements: (S11)    S21     (S12)   (S22)\n")
        else:
            file.write( "! Measurements:  S11     S21     S12     S22\n")

    elif fileformat == 2:
        file.write( "! Measurements: S21\n")
    else:
        file.write( "! Measurements: S11\n")
    file.write( "!\n")

    for x in range( len( f)):
        if fileformat == 1:
            if numparamstowrite == 1:
                file.write( "{:f}   0   0   {:f}    {:f}    0   0   0   0\n".format( f[x], numpy.real(s[x][0]), numpy.imag(s[x][0])))
            else:
                file.write( "{:f}   {:f}    {:f}    {:f}    {:f}    {:f}    {:f}    {:f}    {:f}\n".format( f[x], numpy.real(s[x][0]), numpy.imag(s[x][0]), numpy.real(s[x][1]), numpy.imag(s[x][1]), numpy.real(s[x][2]), numpy.imag(s[x][2]), numpy.real(s[x][3]), numpy.imag(s[x][3])))
        else:
            file.write( "{:f}   {:f}    {:f}\n".format( f[x], numpy.real(s[x][0]), numpy.imag(s[x][0])))

    file.close()



if __name__ == "__main__":
	# pass

    # Test Section
    import numpy
    import rskfd

    filename = "myfile"
    extension = ".fres"

    # Create test file
    n= 101
    if extension.lower().find( ".s2p")>=0:
        numparams = 4
    else:
        numparams = 1
    f = numpy.linspace( 1e7, 1e10, num = n)
    #s = numpy.random.rand( n) +1j* numpy.random.rand( n)
    s = 2* ((numpy.random.rand( n, numparams)-.5) + 1j*2*( numpy.random.rand( n, numparams)-.5))
    rskfd.WriteSnP( s, f, filename + "-ri-backup" + extension, inputformat="ri")
    
    # Test conversion
    s,f = rskfd.ReadSnP( filename + "-ri-backup" + extension)
    rskfd.WriteSnP( s, f, filename + "-dbmag" + extension, inputformat="ri", outputformat="dbmag")
    s,f = rskfd.ReadSnP( filename + "-dbmag" + extension)
    rskfd.WriteSnP( s, f, filename + "-mag" + extension, inputformat="ri", outputformat="mag")
    s,f = rskfd.ReadSnP( filename + "-mag" + extension)
    rskfd.WriteSnP( s, f, filename + extension, inputformat="ri")



# -*- coding: utf-8 -*-
"""
Created on Sat Jul 11 2020

(C) 2020, Florian Ramian
"""


def CreateWGNSignal( NoisePower=-40, RecordLength=4096, BandLimit=1.0 ):
    # function returns white Gaussian noise baseband signal
    #  (Gaussian on I and Gaussian on Q, Rayleigh distributed on magnitude)
    # NoisePower:   mean power in dBm
    # RecordLength: Record length in samples
    # BandLimit: limit noise power to BandLimit relative to sampling rate; .8 is a good choice when signal shall be replayed on a generator
    # returns the noise vector (scaled in Volts)

    import numpy
    import logging

    # dBm to Volts^2, 10^(np/10)/1000*50; divided by 2, because of I and Q
    std_dev = numpy.sqrt( numpy.power( 10, NoisePower/10)/40)
    data_real = numpy.random.normal( 0, std_dev, RecordLength)
    data_imag = numpy.random.normal( 0, std_dev, RecordLength)
    data = data_real + 1j * data_imag

    if BandLimit < 1:
        data = LowPassFilter( data, BandLimit)
    
    logging.info( "WGN vector length: "+str(RecordLength)+"; power: "+str(numpy.log10(numpy.mean(numpy.square(numpy.abs(data)))/50)*10+30)+" dBm")
    
    return data

def LowPassFilter( inData, RelativeBandwidth=.8, FilterTaps=100, KeepPower=False):
    # function applies a filter to inData with relative bandwidth
    # inData:               input data vector
    # RelativeBandwidth:    bandwidth of low pass ]0..1[
    # FilterTaps:           number of taps to be used for the filter
    # KeepPower:            if true, output signal is scaled so it has the same mean power as before
    # returns the filtered inData
    
    import numpy
    from scipy import signal
    import logging

    if( RelativeBandwidth<=0)&( RelativeBandwidth>=1):
        logging.error("Relative bandwidth must be >0 and <1!")
        return

    # filter noise, design filter first
    b = signal.firwin( FilterTaps, RelativeBandwidth)
    out = signal.filtfilt( b, 1, inData)

    if KeepPower:
        inpower = numpy.mean( numpy.power( numpy.abs(inData), 2))
        outpower = numpy.mean( numpy.power( numpy.abs(out), 2))
        out = out / outpower * inpower

    return out



if __name__ == "__main__":
    # execute only if run as a script
    pass


# -*- coding: utf-8 -*-
"""
Created on Sun Dec 12 2021

(C) 2021, Rohde&Schwarz, ramian
"""

import rskfd

def ConvertIiQq2Wv():
    '''
    Test
    '''
    # there is no sampling rate in an iqw file
    fs = 320e6
    data = rskfd.ReadIqw( r'\\myserver\signal\higher_qams.iiqq', iqiq=False)
    rskfd.WriteWv( data, fs, r'\\myserver\signal\higher_qams.wv')
    rskfd.WriteBin( data, fs, r'\\myserver\signal\higher_qams.bin')
    rskfd.WriteIqTar( data, fs, r'\\myserver\signal\higher_qams.iq.tar')



def ReadFile( filename):
    '''
    Test
    '''
    data,fs = rskfd.ReadWv( filename)
    print( f'RMS power in file: {rskfd.MeanPower( data)} dBm, peak power: {rskfd.MeanPower( data)} dBm.\n')
    rskfd.WriteWv( data, fs, 'myfilename.wv')



def ReadWvTest():
    '''
    Test the wv reading routine (tags!!)
    '''
    iq, fs = rskfd.ReadWv(r'\\myserver\signal\higher_qams.wv')


def ReadIqTar64bitTest():
    '''
    Test the wv reading routine (tags!!)
    '''
    # iq,fs = rskfd.ReadIqTar(r'C:\Users\ramian\Documents\k18\testing\AmptoolsMeas.iq.tar')
    # iq = rskfd.ReadIqw(r'C:\Users\ramian\Downloads\iq_data.complex.float64', 'float64')
    iq1, fs = rskfd.ReadIqTar(r'C:\Users\ramian\Documents\wideband_vna\KeysightApproach\Testsignal\FileMay.iq.tar')

    rskfd.WriteIqTar(iq1, fs, 'float32test.iq.tar', DataType='float32')
    rskfd.WriteIqTar(iq1, fs, 'float64test.iq.tar', DataType='float64')
    rskfd.WriteIqTar(iq1, fs, 'int8.iq.tar', DataType='int8')
    rskfd.WriteIqTar(iq1, fs, 'int16.iq.tar', DataType='int16')
    rskfd.WriteIqTar(iq1, fs, 'int32.iq.tar', DataType='int32')
    iq2, fs2 = rskfd.ReadIqTar('float32test.iq.tar')
    iq3, fs3 = rskfd.ReadIqTar('float64test.iq.tar')
    iq4, fs4 = rskfd.ReadIqTar('int8.iq.tar')
    iq5, fs4 = rskfd.ReadIqTar('int16.iq.tar')
    iq6, fs4 = rskfd.ReadIqTar('int32.iq.tar')

    error1 = 0
    error2 = 0
    error3 = 0
    error4 = 0
    error5 = 0
    iqvectorlength = len(iq1)
    for n in range(iqvectorlength):
        error1 = error1 + abs(iq1[n] - iq2[n])
        error2 = error2 + abs(iq1[n] - iq3[n])
        error3 = error3 + abs(iq1[n] - iq4[n])
        error4 = error4 + abs(iq1[n] - iq5[n])
        error5 = error5 + abs(iq1[n] - iq6[n])

    error1 = error1 / iqvectorlength
    error2 = error2 / iqvectorlength
    error3 = error3 / iqvectorlength
    error4 = error4 / iqvectorlength
    error5 = error5 / iqvectorlength

    if error1 > 1e-20:
        print(f'float32 not ok (error {error1})!')

    if error2 > 1e-40:
        print(f'float64 not ok (error {error2})!')

    if error3 > 2/2**8:
        print(f'int8 not ok (error {error3})!')

    if error4 > 2/2**16:
        print(f'int16 not ok (error {error4})!')

    if error5 > 2/2**32:
        print(f'int32 not ok (error {error5})!')

    rskfd.ShowLogFFT(iq1)
    pass


if __name__ == "__main__":
	#pass
    #ReadWvTest()
    ReadIqTar64bitTest()




import setuptools

with open( "README.md", "r") as filein:
  long_description = filein.read()

setuptools.setup(
  name = 'rskfd',
  #packages = ['rskfd','rskfd.remote_control','rskfd.iq_data_handling','rskfd.signal_generation','rskfd.helper'],
  #packages = setuptools.find_packages(),
  packages = setuptools.find_namespace_packages(),
  version = '0.5.1',
  license='MIT',
  description = 'Python Package for Instrument Control and Data Handling',
  long_description = long_description,
  long_description_content_type = "text/markdown",
  author = 'Florian Ramian', 
  author_email = 'gitlab@ramian.eu',
  url = 'https://gitlab.com/ramian/rskfd',   # gitlab
  download_url = 'https://gitlab.com/ramian/rskfd/',
  keywords = ['INSTRUMENT CONTROL', 'I/Q DATA','WV','IQ.TAR'],
  install_requires=[
          'numpy',
          'scipy'
      ],
  python_requires='>=3.0',
  classifiers=[
    'Development Status :: 4 - Beta',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',      # Define that your audience are developers
    'Topic :: Scientific/Engineering',
    'Topic :: Scientific/Engineering :: Electronic Design Automation (EDA)',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
  ],
)



# -*- coding: utf-8 -*-
"""
Created on Tue Mar 26 08:13:26 2019

@author: RAMIAN
"""

from instrumentRS import instrumentRS
from instrumentRSExceptions import FileLoadError


class instrumentRS_VSE(instrumentRS):      
    """Python class implementing functionality for Rohde & Schwarz VSE Vector Signal Explorer software.
    It is based on the instrumentRS class for socket connectivity of R&S instruments."""
    #Store all measurement objects in this dictionary
    __MeasurementList = {}
    __PersistentChannels = None
    __OverWriteChannels = None
    __CurrentChannel = "Spectrum"
    
    def __init__(self, IpAddress,Port=5025, timeout=2, PersistentChannels = True, OverWriteChannels = True):
        super(instrumentRS_VSE,self).__init__( IpAddress, Port, timeout)
        self.__PersistentChannels = PersistentChannels
        self.__OverWriteChannels = OverWriteChannels
        
    def ChannelSelect( self, channelname):
        if self.__CurrentChannel != channelname:
            self.__CurrentChannel = channelname
            self.Write( "INST:SEL '"+ self.__CurrentChannel+"'")

    def LoadFile( self, filename, channelname=""):
        if channelname != "":
            self.ChannelSelect( channelname)
        self.Write("INST:BLOC:CHAN:SOUR FILE")
        if ".iq.tar" in filename.lower():
            self.Write("INST:BLOC:CHAN:FILE:IQT \""+filename+"\",1")
        else:
            raise FileLoadError( "File format not supported")
    
    
def main():
    #for testing only, commented out are some things you can test!
    #pass

    #import time
    vse=instrumentRS_VSE("127.0.0.1",PersistentChannels =False)
    #vse.StartLogging("test.txt")
    vse.Connect()
    print(vse.GetID())
    
    vse.GetSystErr()
    vse.Close()
    
    
if __name__ == "__main__":
    # execute only if run as a script
    main()    
